/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */


#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/types.h>

#include "libsockmand.h"

int main(int argc,char**argv)
{
    if(argc != 4){
        printf("usage: %s ip port filename\n", argv[0]);
        return -1;
     }
    
    
    char *addr = argv[1];
    short port = atoi(argv[2]);
    
     if(port < 0 || port > 65536){
	printf("usage: %s ip port filename\n", argv[0]);
	return 0;
    }    

    char* filename = argv[3];
    
    struct sockaddr_in serv_addr;
    serv_addr.sin_family = AF_INET;
    serv_addr.sin_addr.s_addr = inet_addr(addr);
    serv_addr.sin_port = htons(port);

    if(serv_addr.sin_addr.s_addr == -1){
	printf("usage: %s ip port filename\n", argv[0]);
	return 0;
    }    


    FILE *file;
    char *buffer;
    unsigned long fileLen;
    
    //Open file
    file = fopen(filename, "rb");
    if (!file)
    {
        fprintf(stderr, "Unable to open file %s\n", filename);
        return 0;
    }
    
    //Get file length
    fseek(file, 0, SEEK_END);
    fileLen=ftell(file);
    fseek(file, 0, SEEK_SET);
    
    //Allocate memory
    buffer=(char *)malloc(fileLen+1);
    if (!buffer)
    {
        fprintf(stderr, "Memory error!\n");
        fclose(file);
        return 0;
    }
    
    //Read file contents into buffer
    fread(buffer, fileLen, 1, file);
    fclose(file);
    int size = fileLen;
    printf("Size of file %d\n", size);
 
    int sd = sm_socket(AF_INET,SOCK_STREAM,0,0);

    sm_connect(sd, (struct sockaddr *) &serv_addr,sizeof(serv_addr));
    sm_write(sd, &size, sizeof(int));

    int remaining = size;
    char *b = buffer;
    while(remaining > 0){
        int r = sm_write(sd, b, remaining);
        b+=r;
        remaining -= r;
        
    }
    sm_close(sd);
    free(buffer);
    return 0;
}

