/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */


#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <stdlib.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <string.h>

#include "libsockmand.h"

#define BUFSIZE 1024

int main(int argc,char**argv)
{
    if(!(argc == 3 || argc == 2)){
        printf("usage: %s ip port\n", argv[0]);
        printf("or usage: %s uuid\n", argv[0]);
        return -1;
    }
    
    int sd;
    if(argc == 3){
        
        char *addr = argv[1];
        short port = atoi(argv[2]);
        
        if(port < 0 || port > 65536){
            printf("usage: %s ip port\n", argv[0]);
            printf("or usage: %s uuid\n", argv[0]);
            return 0;
        }    
        
        struct sockaddr_in serv_addr;
        serv_addr.sin_family = AF_INET;
        serv_addr.sin_addr.s_addr = inet_addr(addr);
        serv_addr.sin_port = htons(port);
        
        if(serv_addr.sin_addr.s_addr == -1){
            printf("usage: %s ip port filename", argv[0]);
            return 0;
        }    
        
        sd = sm_socket(AF_INET,SOCK_STREAM,0,0);
        sm_connect(sd, (struct sockaddr *) &serv_addr,sizeof(serv_addr));
        if(sd == -1){
            printf("Unable to connect\n");
            return -1;
        }
        printf("Connected socket with UUID: %d\n", sm_getuuid(sd));
    }
    
    else if(argc == 2){
        int uuid = atoi(argv[1]);
        sd = sm_rebuildsocket(uuid);
         if(sd == -1){
            printf("Unable to rebuid socket\n");
            return -1;
        }
    }
    printf("You can now write messages to the legacy app\n");
    char buffer[BUFSIZE];
    while(1){
        
        if(fgets( buffer, BUFSIZE, stdin ) != 0){
            sm_write(sd, buffer, strlen(buffer));
        }
    }
    return 0;
}

