#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>



int main(int argc, char **argv){
    struct timeval start, prev, now, end;
    long elapsed_utime;    /* elapsed time in microseconds */
    long elapsed_seconds;  /* diff between seconds counter */
    long elapsed_useconds; /* diff between microseconds counter */
    
    if(argc != 6){
        fprintf(stderr, "Usage: %s <MCAP IP address> <dest port number> <number of seconds> <packets per seconds> <packetsize>\n", argv[0]);
        return 0;
    }
    
    unsigned int toaddr = inet_addr(argv[1]);
    if(toaddr==-1){
        printf("Could not parse <MCAP IP address>\n");
        return 0;
    }
    
    int destport=atoi(argv[2]);
    
    if(destport==-1){
        printf("Could not parse <dest port number>\n");
        return 0;
    }
    
    int srcport=9999;
    
    int seconds = atoi(argv[3]);
    if(seconds==-1){
        printf("Could not parse <number of seconds>\n");
        return 0;
    }
    int packetrate =  atoi(argv[4]);
    
    if(packetrate==-1){
        printf("Could not parse <packets per seconds>\n");
        return 0;
    }
    
    int buflen = atoi(argv[5]);
    
    if(buflen==-1){
        printf("Could not parse <packetsize>\n");
        return 0;
    }
    
    int totalpackets = packetrate*seconds;
    char buf[buflen];
    long utimebetweensend = 1000000/packetrate;
    
    
    printf("Sending %d packets\n", totalpackets);
    printf("%d packets / second\n", packetrate);
    printf("Sending for a total of %d seconds\n", seconds);
    printf("Microseconds between each packet %ld\n", utimebetweensend);
    
    
    int s = socket(AF_INET,SOCK_DGRAM,0);
    
    struct sockaddr_in servaddr;    
    bzero(&servaddr,sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    
    servaddr.sin_addr.s_addr = toaddr;
    servaddr.sin_port = htons(destport);
    
    long left = 0;
    int sendtosize = sizeof(servaddr);
    
    int i;
    gettimeofday(&start, NULL);
    sendto(s, buf, buflen, 0, (struct sockaddr *)&servaddr,sendtosize);        
    gettimeofday(&prev, NULL);
    
    
    for(i=0;i<totalpackets-1;i++)
    {
        
        while(1){
            gettimeofday(&now, NULL);
            elapsed_seconds  = now.tv_sec  - prev.tv_sec;
            elapsed_useconds = now.tv_usec - prev.tv_usec;
            elapsed_utime = (elapsed_seconds) * 1000000 + elapsed_useconds;
            left = elapsed_utime - utimebetweensend;
            if(left > 0)
                break;
        }
        sendto(s, buf, buflen, 0, (struct sockaddr *)&servaddr,sendtosize);
        memcpy(&prev,&now,sizeof(struct timeval)); 
        prev.tv_usec -= left;
    }
    
    gettimeofday(&end, NULL);
    
    printf("\nDONE\n\n");
    
    elapsed_seconds  = end.tv_sec  - start.tv_sec;
    elapsed_useconds = end.tv_usec - start.tv_usec;
    elapsed_utime = (elapsed_seconds) * 1000000 + elapsed_useconds;
    
    double secs = elapsed_utime/1000000.0;
    
    printf("Sent %d packets\n", totalpackets);
    double actualrate = (double)totalpackets/secs; 
    printf("Actual packets / second %f\n",actualrate);
    printf("Elapsed time = %ld microseconds\n", elapsed_utime);
    printf("Elapsed time = %f seconds\n", secs);
    close(s);
    return 0;
}




