/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */



#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <signal.h>

#define BUFLEN 1500

int npacksreceived;

struct timeval start, end;

void signal_handler(int sig){
    printf("Receieved %d packets\n", npacksreceived);
    
    long elapsed_utime;    /* elapsed time in microseconds */
    long elapsed_seconds;  /* diff between seconds counter */
    long elapsed_useconds; /* diff between microseconds counter */
    
    elapsed_seconds  = end.tv_sec  - start.tv_sec;
    elapsed_useconds = end.tv_usec - start.tv_usec;
    elapsed_utime = (elapsed_seconds) * 1000000 + elapsed_useconds;
    
    double secs = elapsed_utime/1000000.0;
    
    double actualrate = (double)npacksreceived/secs; 
    printf("Actual packets / second %f\n",actualrate);
    printf("Elapsed time = %ld microseconds\n", elapsed_utime);
    printf("Elapsed time = %f seconds\n", secs);
    exit(EXIT_SUCCESS);
}

int main(int argc, char **argv){
    npacksreceived = 0;
    signal(SIGINT, &signal_handler);
    struct sockaddr_in   si_local;
    int sd;
    int port;
    char buf[BUFLEN];
    
    if(argc!=2){
        fprintf(stderr, "Usage: %s <port number>\n", argv[0]);
        exit(EXIT_FAILURE);
    }
    
    port=atoi(argv[1]);

    if ((sd=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP))==-1){
        perror("socket");
        exit(EXIT_FAILURE);
    }
    
    memset((char *) &si_local, 0, sizeof(si_local));
    si_local.sin_family       =  AF_INET;
    si_local.sin_port         =  htons(port);
    si_local.sin_addr.s_addr  =  htonl(INADDR_ANY);
    if (bind(sd, (const struct sockaddr *)&si_local, sizeof(si_local))==-1)
    {
        perror("bind");
        exit(EXIT_FAILURE);
    }
    int first = 1;
    while(1)
    {
        if (recvfrom(sd, buf, BUFLEN, 0, 0, 0)==-1)
        {
            perror("recvfrom()");
            exit(EXIT_FAILURE);
        }
        else
        {            
            npacksreceived++;
            if(first==1){
                first=0;
                gettimeofday(&start, NULL);
            }
            else{
                gettimeofday(&end, NULL);
            }
        }
    }
}
