/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */


#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/select.h>
#include <unistd.h>

#include "core.h"
#include "sockmandapi.h"
#include "libpcaphandler.h"
#include "tunnel.h"
#include "tcpcont.h"
#include "signalhandler.h"
#include "signal.h"
#include "ui.h"



static int running = 1;

// OS signal handler
void signal_handler(int sig){
    switch(sig)
    {
        case SIGINT:
            //Stop main event loop
            running = 0;
            break;
            
        case SIGTERM:
            //Stop main event loop
            running = 0;
            break;
    }
}


//File descriptor set used by select
int max_fd;
fd_set read_set;


/* Called by TCP conttroller to determine 
 * if ack will be piggybacked
 */
int willpiggyback(int appsockid){
    return FD_ISSET(signalsockets[appsockid].fd, &read_set);
}

void build_select_list() { 
    //Must zero out each time
    FD_ZERO( &read_set );
        
    //add the tunnel to the read set
    FD_SET( tunnel_sockfd, &read_set ); 
    max_fd = tunnel_sockfd;
    
    //Add the mcap to the read set
    FD_SET( pcap_fd, &read_set ); 
    if(pcap_fd > max_fd)
        max_fd = pcap_fd;
    
    //Add the signal listener to the read set
    FD_SET( listenfd, &read_set ); 
    if(listenfd > max_fd)
        max_fd = listenfd;
    
    //User ui for testing purposes
    if(daemonized == 0){
        FD_SET(STDIN_FILENO, &read_set );             
        if(STDIN_FILENO > max_fd)
            max_fd = STDIN_FILENO;
    }
    
    //App ipc
    FD_SET(app_sockfd, &read_set );             
    if(app_sockfd > max_fd)
        max_fd = app_sockfd;
    
    //Add the active signal listeners to the read set
    int i;
    for(i = 0; i < nsignals; i++){
        if(signalsockets[i].active){
            FD_SET( signalsockets[i].fd, &read_set );
            if(signalsockets[i].fd > max_fd)
                max_fd = signalsockets[i].fd;
        }
    }
    
    //Add active app sockets
    for(i = 0; i < napps; i++){
        if(appsockets[i].active){
            FD_SET( appsockets[i].unixfd, &read_set );
            if(appsockets[i].unixfd > max_fd)
                max_fd = appsockets[i].unixfd;
        }
    }
}


void handle_events( ){
    //optimization: use libevent
    fprintf(stderr,"Starting main event loop\n");
    while( running ){  
        build_select_list();
        struct timeval timeout;

        timeout.tv_sec = 0;
        timeout.tv_usec = CLOCKGRANULARITY * 1000;
        
        int retval = select( max_fd + 1, &read_set, 0, 0, &timeout );
        
        switch( retval ){
            case -1 :
                break;
            case 0 :
                checktcptimeout();
                break;
            default :
                //Check if tunnel packets has arrived
                if(FD_ISSET(tunnel_sockfd, &read_set)){ 
                    read_tunneled_packet( );
                }
                //Check if packets have been captured
                if(FD_ISSET(pcap_fd, &read_set)){
                    read_raw_packet( );
                }
                //Check timeouts
                checktcptimeout();
                
                //Check if a new signal connection is here
                if(FD_ISSET(listenfd, &read_set)){
                    handlesignalconnection();   
                }
                
                //Check for keyboard
                if(daemonized == 0){
                    if(FD_ISSET(STDIN_FILENO, &read_set)){
                        handle_keyboard( );
                    }
                }
                
                //Check for local IPC
                if(FD_ISSET(app_sockfd, &read_set)){
                    handleappconnection();
                }
                int i;
                //Check for new data from apps
                for(i = 0; i < napps; i++){
                    if(appsockets[i].active)
                        if(FD_ISSET(appsockets[i].unixfd, &read_set)) {
                            handleapp(appsockets[i].unixfd);
                        }
                }         
                
                //Check for any new signals
                for(i = 0; i < nsignals; i++){
                    if(signalsockets[i].active)
                        if(FD_ISSET(signalsockets[i].fd, &read_set)) {
                            handlesignal(signalsockets[i].fd);
                        }
                }
                break;
        }
    }
}
