/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#define ETHERNETMTU 1500
#define MAXTCPPAYLOAD ETHERNETMTU-20-20-8
#define MAXUDPPAYLOAD ETHERNETMTU-20-8-8


struct ipheader {
    unsigned char ip_hl:4, ip_v:4; 
    unsigned char ip_tos;
    unsigned short int ip_len;
    unsigned short int ip_id;
    unsigned short int ip_off;
    unsigned char ip_ttl;
    unsigned char ip_p;
    unsigned short int ip_sum;
    unsigned int ip_src;
    unsigned int ip_dst;
}; /* total ip header length: 20 bytes (=160 bits) */

struct bufferedpacket{
    struct bufferedpacket* next;
    struct ipheader* ip; //Pinter to the packet
};

struct udpheader {
    unsigned short int uh_sport;
    unsigned short int uh_dport;
    unsigned short int uh_len;
    unsigned short int uh_check;
}; /* total udp header length: 8 bytes (=64 bits) */


struct tcp {
    unsigned short int th_sport;   /* source port */
    unsigned short int th_dport;   /* destination port */
    unsigned int th_seq;     /* sequence number */
    unsigned int th_ack;     /* acknowledgement number */
    //unsigned char th_offx2;    /* data offset, rsvd */
    unsigned char      tcph_reserved:4, tcph_offset:4;
    
    #define TH_OFF(th)  (((th)->th_offx2 & 0xf0) >> 4)
    unsigned char th_flags;
    #define TH_FIN 0x01
    #define TH_SYN 0x02
    #define TH_RST 0x04
    #define TH_PUSH 0x08
    #define TH_ACK 0x10
    #define TH_URG 0x20
    #define TH_ECE 0x40
    #define TH_CWR 0x80
    #define TH_FLAGS (TH_FIN|TH_SYN|TH_RST|TH_ACK|TH_URG|TH_ECE|TH_CWR)
    unsigned short int th_win;     /* window */
    unsigned short int th_sum;     /* checksum */
    unsigned short int th_urp;     /* urgent pointer */
};