/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <arpa/inet.h>


#include "core.h"
#include "ipcont.h"
#include "tcpcont.h"
#include "udpcont.h"
#include "headers.h"
#include "utils.h"



void build_ip_packet(struct ipheader *ip, int protocol, unsigned int srcaddr, unsigned int destaddr, int payloadlen){
    ip->ip_hl = 5; 
    ip->ip_v = 4; 
    ip->ip_tos = 16; 
    ip->ip_len = sizeof(struct ipheader) + payloadlen; 
    ip->ip_id = 0;
    ip->ip_ttl = 64; 
    ip->ip_p = protocol;
    ip->ip_src = srcaddr;
    ip->ip_dst = destaddr;
}


void read_ip_packet(struct ipheader *ip){
    if(is_tcp(ip)){
        tcpread(ip);
    }
    else if(is_udp(ip)){
        udpread(ip);
    }
    else{
        printf("Unsupported transport layer protocol\n");
    }
}

int send_ip_packet(struct ipheader *ip){
    ip->ip_sum = csum((unsigned short *)ip, ip->ip_len);
    return handle_ip_packet(ip);
}


int is_app_present(struct ipheader *ip){
    if(is_tcp(ip)){
        return tcp_is_app_present(ip);
    }
    
    else if(is_udp(ip)){
        return udp_is_app_present(ip);
    }    
    else{
        printf("Unsupported transport layer protocol\n");
        return -1;
    }
}

unsigned int get_app_location(struct ipheader *ip){
    if(is_tcp(ip)){
        return tcp_get_app_location(ip);
    }
    
    else if(is_udp(ip)){
        return udp_get_app_location(ip);
    }
    
    else{
        printf("Unsupported transport layer protocol\n");
        return -1;
    }
}






