/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <stdlib.h>

#include <pcap.h>
#include <errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/if_ether.h> 

#include "core.h"
#include "libpcaphandler.h"
#include "utils.h"
#include "headers.h"


#include <string.h>
#define SIZE_ETHERNET 14
#define MAX_FILTERSIZE 2048


int pcap_fd;
pcap_t* descr;

//Filters

char filter[MAX_FILTERSIZE];

char errbuf[PCAP_ERRBUF_SIZE];
struct bpf_program fp;      /* hold compiled program     */
bpf_u_int32 maskp;          /* subnet mask               */
bpf_u_int32 netp;           /* ip                        */

void packet_callback(unsigned char *Args, const struct pcap_pkthdr* Pkthdr, unsigned char *Packet){
    struct ipheader *ippacket = (struct ipheader *)(Packet + SIZE_ETHERNET);
    ippacket->ip_len = ntohs(ippacket->ip_len);
    handle_captured_packet(ippacket);
}

void read_raw_packet( ){
    pcap_dispatch(descr, 1, (void *) packet_callback, NULL);
}


int set_filter(){
    fprintf(stderr, "Setting filter '%s'\n", filter);
    if(pcap_compile(descr,&fp,filter,1,netp) == -1){ 
        fprintf(stderr,"Error calling pcap_compile\n"); 
        return(-1); 
    }

    /* set the compiled program as the filter */
    if(pcap_setfilter(descr,&fp) == -1){ 
        fprintf(stderr,"Error setting filter\n"); 
        return(-1); 
    }

    return 0;
}

int libpcaphandler_init(){
    char *dev; 

    dev = pcap_lookupdev(errbuf);
    if(dev == NULL){ 
        fprintf(stderr,"%s\n",errbuf); 
        return(-1); 
    }
    
    pcap_lookupnet(dev,&netp,&maskp,errbuf);

    descr = pcap_open_live(dev,BUFSIZ,0,-1,errbuf);
    if(descr == NULL){ 
        printf("pcap_open_live(): %s\n",errbuf); 
        return(-1); 
    }
    
    pcap_fd =  pcap_get_selectable_fd(descr);
    if(pcap_fd < 0){
        fprintf(stderr,"Error getting pcap selectable\n"); 
        return(-1); 
    }
    
    bzero(filter,MAX_FILTERSIZE);
    memcpy(filter, "tcp dst port 1", 15);
    //Setting a dummy filter to stop all packets from arriving
    set_filter();
   
    printf("pcap init done\n");
    return 0;
}


int libpcaphandler_exit(){
    pcap_close(descr);
    return 0;
}


int register_udp_capture(short dport){
    int port =  ntohs(dport);
    fprintf(stderr, "register_udp_capture %d\n", port);
    if(port > 65535){
        fprintf(stderr,"To large port num %d\n", port);   
        return -1;
    }
    if(strlen(filter)>MAX_FILTERSIZE -30){
        printf("To many filters\n");
        exit(-1);
    }
    char *a= " or udp dst port ";
    strcat(filter, a);
    char b[7];
    sprintf(b, "%d", port );
    strcat(filter, b);

    return set_filter();    
}

void droprstpackets(short sport){
    //Drops reset packets sent from the OS kernel
    int port =  ntohs(sport);
    char comm[1000];
    snprintf(comm, sizeof(comm), "iptables -A OUTPUT -p tcp --sport %d --tcp-flags rst rst -j DROP", port);
    if(system(comm)<0){
        perror("System() ");
        exit(-1);
    }
}

int register_tcp_capture(short dport){
    droprstpackets(dport);
    int port =  ntohs(dport);  
    fprintf(stderr, "register_tcp_capture %d\n", port);
     if(port > 65535){
        fprintf(stderr,"To large port num %d\n", port);   
        return -1;
    }
    if(strlen(filter)>MAX_FILTERSIZE -30){
        printf("To many filters\n");
        exit(-1);
    }  
    char *a= " or tcp dst port ";
    strcat(filter, a);
    char b[7];
    sprintf(b, "%d", port );
    strcat(filter, b);

    return set_filter();    
}