/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netinet/ip.h>

#include "libsockmand.h"
#include "appmessages.h"
#define HIGHESTFD 1024 
//Must be increased if a fd exceeds 1024, unlikely

int socketprotocol[HIGHESTFD];

//Used to recrate this socket
int uuid_of_fd[HIGHESTFD];

int readint(int fd){
    int ret = -1;
    read(fd, &ret, sizeof(int));
    return ret;
}

int sm_socket(int domain, int type, int protocol, int mcappolicy){
    //Verify arguments
    if(domain != AF_INET)
        return -1;    
    if(type == SOCK_STREAM){
        if(!(protocol == 0 || protocol == IPPROTO_TCP))
            return -1;
        if(protocol == 0)
            protocol = IPPROTO_TCP;
    }
    else if(type == SOCK_DGRAM){
        if(!(protocol == 0 || protocol == IPPROTO_UDP))
            return -1;
        if(protocol == 0)
            protocol = IPPROTO_UDP;
    }
    else{
        return -1;
    }
    
    int s, len;
    struct sockaddr_un remote;
    
    if((s = socket(AF_UNIX, SOCK_STREAM, 0)) == -1){
        perror("socket");
        return -1;
    }
    
    remote.sun_family = AF_UNIX;
    strcpy(remote.sun_path, SOCK_PATH);
    len = strlen(remote.sun_path) + sizeof(remote.sun_family);
    if(connect(s, (struct sockaddr *)&remote, len) == -1){
        perror("connect");
        return -1;
    }
    
    struct socketmsg msg;
    msg.msgtype = SOCKET;
    msg.protocol = protocol;
    msg.mcappolicy = mcappolicy;
    
    write(s, &msg, sizeof(struct socketmsg));
    
    socketprotocol[s] = protocol;
    uuid_of_fd[s] = readint(s);
    return s;
}    

int sm_rebuildsocket(int uuid){
    int s, len;
    struct sockaddr_un remote;
    
    if((s = socket(AF_UNIX, SOCK_STREAM, 0)) == -1){
        perror("socket");
        return -1;
    }
    
    remote.sun_family = AF_UNIX;
    strcpy(remote.sun_path, SOCK_PATH);
    len = strlen(remote.sun_path) + sizeof(remote.sun_family);
    if(connect(s, (struct sockaddr *)&remote, len) == -1){
        perror("connect");
        return -1;
    }
    
    struct rebuildmsg msg;
    msg.msgtype = REBUILD;
    msg.uuid = uuid;
    
    write(s, &msg, sizeof(struct rebuildmsg));
    
    int ret = readint(s);
    if(ret == -1){
        close(s);
        return ret;
    }
    uuid_of_fd[s] = uuid;
    socketprotocol[s] = ret;
    return s;
}

int sm_getuuid(int sockfd){
    return uuid_of_fd[sockfd];
}

//For both
int sm_bind(int sockfd, const struct sockaddr *addr, socklen_t addrlen){
    struct sockaddr_in *addr2 =  (struct sockaddr_in*) addr;
    short port = addr2->sin_port;
    struct bindmsg msg;
    msg.msgtype = BIND;
    msg.port = port;
    write(sockfd, &msg, sizeof(struct bindmsg));
    return readint(sockfd);
}

int sm_close(int sockfd){
    return close(sockfd);
}

//For tcp
int sm_connect(int sockfd, const struct sockaddr *addr, socklen_t addrlen){
    if(socketprotocol[sockfd]== IPPROTO_UDP)
        return -1;
    
    struct sockaddr_in *addr2 =  (struct sockaddr_in*) addr;
    unsigned int dest_addr = addr2->sin_addr.s_addr;
    short port = addr2->sin_port;
    
    struct connectmsg msg;
    msg.msgtype = CONNECT;
    msg.dest_addr = dest_addr;
    msg.port = port;
    write(sockfd, &msg, sizeof(struct connectmsg));
    return readint(sockfd);
}

int sm_listen(int sockfd){
    if(socketprotocol[sockfd]== IPPROTO_UDP)
        return -1;
    struct listenmsg msg;
    msg.msgtype = LISTENMSG;
    write(sockfd, &msg, sizeof(struct listenmsg));
    return readint(sockfd);
}

int sm_accept(int sockfd){
    if(socketprotocol[sockfd]== IPPROTO_UDP)
        return -1;
    return readint(sockfd);
}

int sm_read(int fd, void *buf, int count){
    if(socketprotocol[fd]== IPPROTO_UDP)
        return sm_recvfrom(fd, buf, count, 0, 0, 0);
    return read(fd, buf, count);
}

int sm_write(int fd, const void *buf, int count){
    if(socketprotocol[fd]== IPPROTO_UDP)
        return -1;
    
      if(count>MAXWRITESIZE)
        count = MAXWRITESIZE;
    
    struct writemsg *msg = malloc(sizeof(struct writemsg));
    msg->len = count;
    msg->msgtype = WRITE;    
    write(fd, msg, sizeof(struct sendtomsg));
    write(fd, buf, count);
    return readint(fd);
}


int sm_sendto(int fd, const void *buf, int len, int flags, struct sockaddr *dest_addr, socklen_t addrlen){
    if(socketprotocol[fd]== IPPROTO_TCP)
        return -1;
    
    if(len>1500-20-20-8-8)
        return -1; //To large for ethernet mtu
        
    struct sockaddr_in *dstaddr =  (struct sockaddr_in*) dest_addr;
    
    struct sendtomsg *msg = malloc(sizeof(struct sendtomsg));
    msg->msgtype = SENDTO;
    msg->destport = dstaddr->sin_port;
    msg->destaddr = dstaddr->sin_addr.s_addr;
    msg->len = len;
    
    int k = write(fd, msg, sizeof(struct sendtomsg));
    int k2 = write(fd, buf, len); 
    
    if(k!=sizeof(struct sendtomsg) || k2 != len)
        return -1;

    free(msg);
    return len;
}

int sm_recvfrom(int sockfd, void *buf, size_t len, int flags, struct sockaddr *src_addr, socklen_t *addrlen){
    struct recvfrommsg m;
    read(sockfd, &m, sizeof(struct recvfrommsg));
    
    if(flags!=0){
        fprintf(stderr, "Flags not yet suppported, use 0\n");
        return -1;
    }

    //fill in the src addr struct if possible
    if(src_addr != 0 && addrlen !=0 && *addrlen >= sizeof(struct sockaddr_in)){
        struct sockaddr_in *servaddr =  (struct sockaddr_in*) src_addr;
        bzero(servaddr,sizeof(struct sockaddr_in));
        servaddr->sin_family = AF_INET;
        servaddr->sin_addr.s_addr = m.sourceaddr;
        servaddr->sin_port = m.sourceport;
    }
   
    //If we ask for more or equal to what was available
    if(m.len <= len){
        return read(sockfd, buf, m.len);
    }
        
    //If we ask for less than what was available
    else{    
        int ret = read(sockfd, buf, len);
        read(sockfd, buf, m.len-len); //Discard the unread part, this is standard POSIX
        return ret;
    }
}
