/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include <unistd.h>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <stdlib.h>

#include <errno.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/if_ether.h> 

#include "core.h"
#include "utils.h"
#include "headers.h"


#include <string.h>
#define SIZE_ETHERNET 14
#define MAX_FILTERSIZE 2048


//We need two sockets since they are initialized with a protocol
int udps; 
int tcps;


int rawsock_init(){
    udps = socket(PF_INET, SOCK_RAW, IPPROTO_UDP);
    if(udps < 0) {
        perror("socket() error");
        exit(-1);
    }

    int one = 1;
    const int *val = &one;
    
    // First set up for TCP
    if(setsockopt(udps, IPPROTO_IP, IP_HDRINCL, val, sizeof(one)) < 0){
        perror("setsockopt() error, try sudo");
        exit(-1);
    }
    else
        printf("setsockopt() is OK\n");
        tcps = socket(PF_INET, SOCK_RAW, IPPROTO_TCP);
    if(tcps < 0) {
        perror("socket() error, try sudo");
        exit(-1);
    }

    // Set up for UDP
    if(setsockopt(tcps, IPPROTO_IP, IP_HDRINCL, val, sizeof(one)) < 0)
    {
        perror("setsockopt() error");
        exit(-1);
    }
    return 0;
}

int rawsock_exit(){
    close(udps);
    close(tcps);
    return 0;
}


int write_ip_udp_packet(struct ipheader *ippacket){
    struct udpheader *udph = (struct udpheader*) (ippacket + 1);
    struct sockaddr_in sin;
    sin.sin_family = AF_INET;
    sin.sin_port = htons(ntohs(udph->uh_dport));
    sin.sin_addr.s_addr = ippacket->ip_dst;
//     printf("Sending raw udp to %s %d len: %d\n", get_readable_dst(ippacket), ntohs(udph->uh_dport), ntohs(udph->uh_len));
    
    return sendto (udps,
                   ippacket,
                   ippacket->ip_len,
                   0,
                   (struct sockaddr *) &sin,
                    sizeof (sin));
}


int write_ip_tcp_packet(struct ipheader *ippacket){
    struct tcp *tcp = (struct tcp*) (ippacket + 1);
    struct sockaddr_in sin;
    sin.sin_family = AF_INET;
    sin.sin_port = tcp->th_dport;
    sin.sin_addr.s_addr = ippacket->ip_dst;
//      printf("Sending with seq %u and ack %u to %s\n\n", ntohl(tcp->th_seq), ntohl(tcp->th_ack), get_readable_dst(ippacket));

    return sendto (tcps,
                   ippacket,
                   ippacket->ip_len,
                   0,
                   (struct sockaddr *) &sin,
                    sizeof (sin));
}



int write_ip_packet(struct ipheader *ippacket){
    if(is_tcp(ippacket)){
        return write_ip_tcp_packet(ippacket);
    }
    else if(is_udp(ippacket)){
        return write_ip_udp_packet(ippacket);
    }
    else{
        fprintf(stderr,"A non udp or tcp packet was tried sent from rawsockhandler.c\n");   
        return -1;
    }
}



