/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "core.h"
#include "signalhandler.h"
#include "utils.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#define SIGNALPORT 5000

int listenfd;
int nsignals;

struct signalsocket signalsockets[1024]; 


void handlesignal(int fd){
    char buf[sizeof(struct signalheader)];
    int n = read(fd, buf, sizeof(struct signalheader));
    
    if (n <= 0){
        //Disconnected
        int i;
        for(i=0; i < nsignals; i++){
            if(signalsockets[i].fd == fd && signalsockets[i].active)
                signalsockets[i].active = 0;
        }
    }
    else{
        struct signalheader *hdr = (struct signalheader*) buf;
        
        if(hdr->signaltype == MIGRATING_TO){
            handle_signal(buf);
            int ret = 1;
            write(fd, &ret, sizeof(int));
        }
        
        else if(hdr->signaltype == SOCKETSTATE){
            struct socketstate* ss =  (struct socketstate*) &buf;
            
            struct socketstate *s = malloc(sizeof(struct socketstate) + ss->length);
            
            memcpy(s, buf, sizeof(struct signalheader));
            
            char *tmp = ((char*)s)+sizeof(struct socketstate);
            int remaining = ss->length - sizeof(struct signalheader);
            while(remaining > 0){
                int r = read(fd, tmp, remaining);
                remaining -= r;
                tmp+=r;
            }
            handle_signal((char*)s);
            free(s);

            int ret = 1;
            write(fd, &ret, sizeof(int));
        }
        else{
            fprintf(stderr,"False signal receieved\n");
        }
    }
}

int sendsignal(char* signal, int len, unsigned int address){
    int i;
    int found = 0;
    int fd = 0;
    for(i = 0; i < nsignals && !found; i++){
        if(signalsockets[i].client == address && signalsockets[i].active){
            found = 1;
            fd = signalsockets[i].fd;
        }
    }
    
    if(!found){
        newsignalconn(address);
        return sendsignal(signal, len, address);
    }
    else{
        int remaining = len;
        while(remaining > 0){
            int r = write(fd, signal, remaining);
            signal+=r;
            remaining -= r;
        }
        
        int ret = 0;
        read(fd, &ret, sizeof(int));
        return ret;
    }
}

void handlesignalconnection(){
    struct sockaddr_in cliaddr;
    socklen_t clilen;
    clilen = sizeof(cliaddr);
    
    int connfd = accept(listenfd,(struct sockaddr *)&cliaddr,&clilen);
    
    printf("Accepted a new signal connection with fd %d\n",connfd);
    
    signalsockets[nsignals].fd = connfd;
    signalsockets[nsignals].active = 1;
    signalsockets[nsignals].client = cliaddr.sin_addr.s_addr; 
    nsignals++;    
}

void newsignalconn(unsigned int address){
    //Maybe we already have one
    int i;
    int found = 0;
    for(i = 0; i < nsignals && !found; i++){
        if(signalsockets[i].client == address && signalsockets[i].active)
            found = 1;
    }
    
    if(!found){
        
        int sockfd = socket(AF_INET, SOCK_STREAM, 0);
        if (sockfd < 0){
            printf("ERROR opening signalsocket\n");
        }
        else{
            
            
            /* build the server's Internet address */
            struct sockaddr_in serveraddr;
            
            bzero(&serveraddr, sizeof(serveraddr));
            
            serveraddr.sin_family = AF_INET;
            serveraddr.sin_addr.s_addr =  address;
            serveraddr.sin_port = htons(SIGNALPORT);
            
            /* connect: create a connection with the server */
            if (connect(sockfd, (struct sockaddr *)&serveraddr, sizeof(serveraddr)) < 0){ 
                perror("connect: ");
                printf("ERROR connecting signalsocket to %s\n", ip_to_char(address));
                exit(0); //TODO handle annen way
            }
            else{
                printf("Signalsocket set up with fd %d\n", sockfd);
                signalsockets[nsignals].fd = sockfd;
                signalsockets[nsignals].active = 1;
                signalsockets[nsignals].client = address;
                nsignals++;
            }
        }
    }
    else{
        printf("Connection already exists\n");
    }
}


void signalinit(){
    nsignals = 0;
    
    struct sockaddr_in servaddr;
    listenfd = socket(AF_INET,SOCK_STREAM,0); 
    bzero(&servaddr,sizeof(servaddr));
    servaddr.sin_family = AF_INET;
    servaddr.sin_addr.s_addr=htonl(INADDR_ANY);
    servaddr.sin_port=htons(SIGNALPORT);
    bind(listenfd,(struct sockaddr *)&servaddr,sizeof(servaddr));
    setnonblocking(listenfd);
    listen(listenfd,SOMAXCONN); 
}

void signal_exit(){
    close(listenfd);
    int i;
    for(i = 0; i < nsignals; i++){
        if(signalsockets[i].active)
            close(signalsockets[i].fd);
    }
}

