/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "core.h"
#include "utils.h"

void usage(char* argv0){
    printf("\nUsage: sudo %s DEFAULTMCAP [-d]\n", argv0);
    printf("-d starts SOCKMAND as a daemon\n");
    printf("DEFAULTMCAP is the IP address of the default MCAP\n\n");
    
}


int main(int argc,char **argv){
    
    if(argc > 3 || argc <2){
        usage(argv[0]);
        return 0;
    }
    
    unsigned int defaultmcap = 0;

    if(!ipchar_to_ipuint(argv[1], &defaultmcap)){
        printf("\nCould not parse MCAP address, example: 192.168.0.2\n");
        usage(argv[0]);
        return 0;
    }

    int isdaemonized = 0;
    if(argc == 3){
        if(strncmp(argv[2], "-d", 2)==0)
            isdaemonized = 1;
        else{
            printf("Invalid argument\n");
            usage(argv[0]);
            return 0;
        }
    }
    
    printf("Using %s as default MCAP address\n", ip_to_char(defaultmcap));

    if(isdaemonized == 1){
        printf("Starting SOCKMAND as daemon\n");
        daemon(0,0);
    }
    else{
        printf("Starting SOCKMAND, not in daemon mode\n");   
    }
    
    core_init(isdaemonized, defaultmcap);
    core_exit();
    return 0;
}