/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "sockmandapi.h"
#include "udpcont.h"
#include "tcpcont.h"
#include "core.h"
#include "utils.h"
#include "appmessages.h"
#include "migapi.h"

#include <netinet/ip.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <time.h>



int app_sockfd;
int napps = 0;
struct appsocket appsockets[1024]; //rename sockettable



unsigned int get_anchor_address(unsigned int mcappolicy){
    if(mcappolicy == 0)
        return get_my_addr();
    return get_mcap(mcappolicy);
    
}


void appapi_init(){  
    int len;
    struct sockaddr_un local;
    
    if ((app_sockfd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
        perror("socket");
        exit(1);
    }
    
    local.sun_family = AF_UNIX;
    strcpy(local.sun_path, SOCK_PATH);
    unlink(local.sun_path);
    len = strlen(local.sun_path) + sizeof(local.sun_family);
    
    if (bind(app_sockfd, (struct sockaddr *)&local, len) == -1) {
        perror("bind");
        exit(1);
    }
    
    setnonblocking(app_sockfd);
    
    if (listen(app_sockfd, 5) == -1) {
        perror("listen");
        exit(1);
    }
    
    //Change permissions of the file
    char comm[1000];
    snprintf(comm, sizeof(comm), "chmod o+rw %s", SOCK_PATH);
    int r = system(comm);
    if(r<0){
        //todo handle error
    }
    printf("Listening for new app connections\n");
}



void appapi_exit(){
    close(app_sockfd);
    int i;
    for(i = 0; i < napps; i++){
        if(appsockets[i].active)
            close(appsockets[i].unixfd);
    }
}

void handleappconnection(){
    struct sockaddr_un cliaddr;
    socklen_t clilen;
    clilen = sizeof(cliaddr);
    
    int connfd = accept(app_sockfd,(struct sockaddr *)&cliaddr,&clilen);
        
    appsockets[napps].unixfd = connfd;
    appsockets[napps].active = 1;
    napps++;    
}

void sendretvalue(int sd, int retvalue){
    write(sd, &retvalue, sizeof(int));
}


int someport = 50100;

short getsomeport(int protocol){
    //TODO More dynamic
    return htons(someport++);
}


void myconnect(int id, unsigned int dest_addr, short destport){
    if(!appsockets[id].active)
        close(appsockets[id].unixfd);
    
    if(appsockets[id].protocol == IPPROTO_UDP){
        sendretvalue(appsockets[id].unixfd, -1);
    }
    
     
    if(appsockets[id].controlblock == -1){
        appsockets[id].controlblock = tcpbind(get_anchor_address(appsockets[id].mcappolicy),
                                              getsomeport(IPPROTO_TCP), 
                                              id,  
                                              appsockets[id].uuid);
    }
    
    tcpconnect(appsockets[id].controlblock, dest_addr, destport);
}


void connect_done(int appsockid, int ret){
    sendretvalue(appsockets[appsockid].unixfd, ret);
}


void mysocket(int id, int protocol, unsigned int mcappolicy){
    srandom(time(0));
    appsockets[id].uuid = random();
    printf("New socket with UUID %d\n", appsockets[id].uuid);
    appsockets[id].active = 1;
    appsockets[id].controlblock = -1;
    appsockets[id].protocol = protocol;
    appsockets[id].mcappolicy = mcappolicy;
    sendretvalue(appsockets[id].unixfd, appsockets[id].uuid );
}

void rebuildsocket(int id, int uuid){
    fprintf(stderr,"Rebuilding socket %d\n",uuid);
    int cb = tcp_get_cb_by_uuid(uuid);
    if(cb != -1){
        struct connectioninfo * c = (struct connectioninfo *) tcp_get_connection_info_by_uuid(uuid);
        c->sockmandapi_id = id;
        appsockets[id].uuid = uuid;
        appsockets[id].active = 1;
        appsockets[id].controlblock = cb;
        appsockets[id].protocol = IPPROTO_TCP;

        sendretvalue(appsockets[id].unixfd, appsockets[id].protocol);
        tcp_deliverbufferdpackets(cb);
        
    }
    else if((cb = udp_get_cb_by_uuid(uuid)) != -1){
        struct connectioninfo * c = (struct connectioninfo *) udp_get_connection_info_by_uuid(uuid);
        c->sockmandapi_id = id;
        appsockets[id].uuid = uuid;
        appsockets[id].active = 1;
        appsockets[id].controlblock = cb;
        appsockets[id].protocol = IPPROTO_UDP;
        sendretvalue(appsockets[id].unixfd, appsockets[id].protocol);

    }
    else{
        appsockets[id].active = 0;
        sendretvalue(appsockets[id].unixfd, -1);
    }
}

void mybind(int id, short port){
    if(!appsockets[id].active)
        sendretvalue(appsockets[id].unixfd, -1);
    
    else if(appsockets[id].controlblock != -1){
        sendretvalue(appsockets[id].unixfd, -1);
    }
    
    else if(appsockets[id].protocol == IPPROTO_TCP){
        appsockets[id].controlblock = tcpbind(get_anchor_address(appsockets[id].mcappolicy),
                                              port, 
                                              id,  
                                              appsockets[id].uuid);
    }
    
    
    else if(appsockets[id].protocol == IPPROTO_UDP){
        appsockets[id].controlblock = udpbind(id, appsockets[id].uuid, port, get_anchor_address(appsockets[id].mcappolicy));
      

    }
    if(appsockets[id].controlblock == -1){
        sendretvalue(appsockets[id].unixfd, -1);
    }
    else{
        sendretvalue(appsockets[id].unixfd, 0);
    }
}

void deliver_tcp_data(int appsockid, char* buf, int len){  
    int remaining = len;
    while(remaining > 0){
        int r = write(appsockets[appsockid].unixfd, buf, remaining);
        buf+=r;
        remaining -= r;
    }
}

void deliver_udp_data(int appsockid, char* buf, int len, short sourceport, unsigned int sourceaddr){
    if(!appsockets[appsockid].active)
        close(appsockets[appsockid].unixfd);
    
    struct recvfrommsg msg;
    msg.sourceport = sourceport;
    msg.len = len;
    msg.sourceaddr = sourceaddr;
    
    write(appsockets[appsockid].unixfd, &msg, sizeof(struct recvfrommsg));
    write(appsockets[appsockid].unixfd, buf, len);
}

//For tcp
void mywrite(int id, const void *buf, int count){
    if(!appsockets[id].active)
        close(appsockets[id].unixfd);
    
    if(appsockets[id].protocol == IPPROTO_UDP)
        sendretvalue(appsockets[id].unixfd, -1);
    else if(appsockets[id].controlblock == -1)
        sendretvalue(appsockets[id].unixfd, -1);
    else{
        int ret = tcpwrite(appsockets[id].controlblock, buf, count);
        sendretvalue(appsockets[id].unixfd, ret);
    }
}

void mylisten(int id){
    //sjekke at den faktisk er bound
    if(appsockets[id].controlblock == -1){
        sendretvalue(appsockets[id].unixfd, -1);
    }
    else{
        int ret = tcplisten(appsockets[id].controlblock);
        sendretvalue(appsockets[id].unixfd, ret);
    }
}

void doaccept(int id){
    sendretvalue(appsockets[id].unixfd, 0);
}


//For udp
void mysendto(int id, const void *buf, int len, unsigned int destaddr, short destport){
    if(!appsockets[id].active)
        close(appsockets[id].unixfd);
    
    if(appsockets[id].controlblock == -1){
        appsockets[id].controlblock = udpbind(id, appsockets[id].uuid, getsomeport(IPPROTO_UDP), get_anchor_address(appsockets[id].mcappolicy));
    }
    udpwrite(appsockets[id].controlblock, buf, len, destaddr, destport);
}

void register_closed(int appsockid){
    close(appsockets[appsockid].unixfd);
    appsockets[appsockid].active = 0;
}


void register_app_disconnection(int unixfd){
    int i;
    for(i=0; i < napps; i++){
        if(appsockets[i].unixfd == unixfd && appsockets[i].active){
            appsockets[i].active = 0;
            if(appsockets[i].protocol == IPPROTO_TCP){
                sendfinack(appsockets[i].controlblock);
            }
            else if(appsockets[i].protocol == IPPROTO_UDP){
                set_udp_unbound(appsockets[i].controlblock);
            }
        }
    }
    printf("Appsocket disconnected\n");
}

int getidbyunixfd(int unixfd){
    int i;
    for(i = 0; i < napps; i++){
        if(appsockets[i].active && appsockets[i].unixfd == unixfd)
            return i;
    }
    return -1;        
}

void handleapp(int unixfd){
    char buf[sizeof(struct socketmsg)];
    
    int id = getidbyunixfd(unixfd);
    int n = recv(unixfd, buf, sizeof(struct socketmsg), 0); 

    if (n <= 0){
        register_app_disconnection(unixfd);
        return;
    }

    else{
        struct msg* msg = (struct msg*)&buf;
        switch(msg->msgtype){
            case SOCKET :
            {
                struct socketmsg* m = (struct socketmsg*) msg;
                mysocket(id, m->protocol, m->mcappolicy);
            }
            break;
            case BIND :
            {
                struct bindmsg* m = (struct bindmsg*) msg;
                mybind(id, m->port);   
            }
            break;              
            case CONNECT :
            {
                struct connectmsg* m = (struct connectmsg*) msg;
                myconnect(id, m->dest_addr, m->port);        
            }
            break;              
            case SENDTO :
            {

                struct sendtomsg* m = (struct sendtomsg*) msg;
                char* data = malloc(m->len);
                int k = recv(unixfd, data, m->len, 0);
                if(k!=m->len){
                    printf("Error, wrong udp read size\n");
                    free(data);
                    return;
                }   
                mysendto(id, data, m->len, m->destaddr, m->destport);         
                free(data);
            }
            break;
            case WRITE :
            {
                struct writemsg* m = (struct writemsg*) msg;
                char* data = malloc(m->len);
                int n = recv(unixfd, data, m->len, 0);
                mywrite( id, data, n);     
                free(data);
            }
            break;
            case REBUILD :
            {
                struct rebuildmsg* m = (struct rebuildmsg*) msg;
                rebuildsocket(id, m->uuid);         
            }
            break;         
            case LISTENMSG :
            {
                mylisten(id);
            }
            break;   
            default :
                printf("Wrong messagetype %d in appapi.c\n",msg->msgtype );
            break;
        }
    }
}

