/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "ipcont.h"
#include "tcpcont.h"
#include "headers.h"
#include "core.h"
#include "utils.h"
#include "libpcaphandler.h"
#include "sockmandapi.h"

#include <time.h>
#include <stdlib.h>
#include <string.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <limits.h>

int next_tcpcb = 0;
struct tcp_cb tcp_cbs[1024];

int nextnum(unsigned int num, int n){
    return htonl(ntohl(num)+ n % UINT_MAX);
}

int between(int a, int b, int c){
    int aa = ntohl(a);
    int bb = ntohl(b);
    int cc = ntohl(c);
    if(((aa < bb) && (bb <= cc)) || ((cc < aa) && (aa < bb)) || ((bb <= cc) && (cc < aa)))
        return TRUE;
    else
        return FALSE;
}

int getflightsize(int cb){
    int a = ntohl(tcp_cbs[cb].nextseq);
    int b = ntohl(tcp_cbs[cb].lastremoteack);
    if(a<=b){
        return b - a;
    }
    else{
        return UINT_MAX - b + a;
    }
}

int tcp_importstate(struct tcp_cb *c){
    memcpy(&tcp_cbs[next_tcpcb], c, sizeof(struct tcp_cb));
    if(tcp_cbs[next_tcpcb].mcap_address == get_my_addr()){
        register_tcp_capture(tcp_cbs[next_tcpcb].sourceport);
    }
    return next_tcpcb++;
}

void* get_tcp_cb_pointer(int id){
    return &tcp_cbs[id];
}

int get_cb_by_pointer(void* p){
    int i;
    for(i = 0;i < next_tcpcb; i++){
        if(p == get_tcp_cb_pointer(i));
           return i;
    }
    return -1; 
}

int tcp_get_cb_by_uuid(int uuid){
    int i;
    for(i = 0;i < next_tcpcb; i++){
        if(tcp_cbs[i].uuid == uuid)
            return i;
    }
    return -1;
}

void* tcp_get_connection_info_by_uuid(int uuid){
    int i;
    for(i = 0;i < next_tcpcb; i++){
        if(tcp_cbs[i].uuid == uuid)
            return get_tcp_cb_pointer(i);
    }
    return 0;
}



int get_tcp_cb(struct ipheader* ip){
    //takes an incoming packet as arg and returns the corresponding cb
    struct tcp *tcp = (struct tcp *) (ip + 1);
    
    int i;
    for(i = 0; i < next_tcpcb; i++){
        if(tcp_cbs[i].status != TCPCLOSED){
            if(ip->ip_dst == tcp_cbs[i].mcap_address &&
                tcp->th_dport == tcp_cbs[i].sourceport)
                return i;            
        }
    }
    
    return -1;
}

int tcp_is_app_present(struct ipheader *ip){
    int cb = get_tcp_cb(ip);
    if(cb == -1)
        return 0;
    return is_my_address(tcp_cbs[cb].current_address);
}

unsigned int tcp_get_app_location(struct ipheader *ip){
    return tcp_cbs[get_tcp_cb(ip)].current_address;
}


void sendclosetoapp(int cb){
   register_closed(tcp_cbs[cb].sockmandapi_id);       
}

void freedatastructures(int cb){
    //implement
}


void build_tcp_packet(struct tcp* tcp,unsigned short srcprt, unsigned short dstprt, unsigned int seq,unsigned int ack){    
    tcp->th_sport = srcprt;
    tcp->th_dport = dstprt;
    tcp->th_seq = seq;
    tcp->th_ack = ack;
    tcp->tcph_offset = 5; //
    tcp->th_win = htons(65535); //Not bound by any buffer
    tcp->th_flags = 0; //Set later
    tcp->th_sum = 0; //Set later
    tcp->th_urp = 0;
}

void setflag(struct tcp* tcp, short flag){
    tcp->th_flags = tcp->th_flags | flag;
}

void ack(int sd, unsigned short int nbytes){
    
    tcp_cbs[sd].lastack = nextnum(tcp_cbs[sd].lastremoteseq, nbytes);
    
    if(willpiggyback(tcp_cbs[sd].sockmandapi_id)){
        printf("Piggbacking!");
        return;
    }
    
    char buffer[ETHERNETMTU]; 
    bzero(&buffer, ETHERNETMTU);
    
    struct ipheader *ip = (struct ipheader *) buffer; 
    struct tcp *tcp = (struct tcp *) (ip + 1);
    
    
    build_ip_packet(ip, 
                    IPPROTO_TCP, 
                    tcp_cbs[sd].mcap_address, 
                    tcp_cbs[sd].dest_address, 
                    sizeof(struct tcp));
    
    
    build_tcp_packet(tcp, 
                     tcp_cbs[sd].sourceport, 
                     tcp_cbs[sd].dest_port, 
                     tcp_cbs[sd].nextseq,
                     tcp_cbs[sd].lastack);
    
    
    setflag(tcp, TH_ACK);
    tcp->th_sum = tcp_checksum(ip, tcp,0);
    send_ip_packet(ip); 
}

void tcp_deliverbufferdpackets(int cb){
    struct bufferedpacket* t = tcp_cbs[cb].bufferedpacketsfirst;
    if(t==0)
        return;
    
    while(t!=0){
        struct ipheader* ip = t->ip;
        struct tcp* tcp = (struct tcp*) (ip +1);
        unsigned short int payloadlen = ntohs(ip->ip_len) - (sizeof (struct ipheader)) - (tcp->tcph_offset * 4);
        char * data = ((char*)tcp) + (tcp->tcph_offset * 4);
        deliver_tcp_data(tcp_cbs[cb].sockmandapi_id, data, payloadlen);
        struct bufferedpacket* tmp = t->next;
        free(ip);
        free(t);
        t=tmp;
    } 
    
    //Ack the packets we delivered to the application
    //The ack number is already set
    char buffer[ETHERNETMTU]; 
    bzero(&buffer, ETHERNETMTU);
    
    struct ipheader *ip = (struct ipheader *) buffer; 
    struct tcp *tcp = (struct tcp *) (ip + 1);
    
    
    build_ip_packet(ip, 
                    IPPROTO_TCP, 
                    tcp_cbs[cb].mcap_address, 
                    tcp_cbs[cb].dest_address, 
                    sizeof(struct tcp));
    
    
    build_tcp_packet(tcp, 
                     tcp_cbs[cb].sourceport, 
                     tcp_cbs[cb].dest_port, 
                     tcp_cbs[cb].nextseq,
                     tcp_cbs[cb].lastack);
    
    
    setflag(tcp, TH_ACK);
    tcp->th_sum = tcp_checksum(ip, tcp,0);
    send_ip_packet(ip); 
}

void close_connection(int cb){
    if(tcp_cbs[cb].status == MIGRATEDAWAY)
        return;
    
    tcp_cbs[cb].status = TCPCLOSED;
    sendclosetoapp(cb);
    freedatastructures(cb);
}

void addtimeout(int id, int type, struct ipheader* ip){
    struct timeouts *t = malloc(sizeof(struct timeouts));
    t->resent_times = 0;
    t->packettype = type;
    if(type == DATA){
        t->timesout = getmillis()+ tcp_cbs[id].RTO;
    }
    else if(type == SYN){
        t->timesout = getmillis()+ tcp_cbs[id].RTO;
    }
    else{
        printf("False timeout type\n");
        return;
    }
    
    t->next = 0;
    t->ip = malloc(ip->ip_len);
    memcpy(t->ip, ip, ip->ip_len);
    
    if(tcp_cbs[id].packets == 0){
        tcp_cbs[id].packets = t;
    }
    
    else{
        struct timeouts* last = tcp_cbs[id].packets;
        while(last->next != 0){
            last=last->next;            
        }
        
        last->next = t;
    }
}


int tcpwrite(int cb, const void *buf, int totallen){
    if(tcp_cbs[cb].status != ESTABLISHED){
        return -1;
    }
    int len = totallen;
    int maxsize = SMSS;

    if(totallen > maxsize)
        len = maxsize;
    len = min(len, tcp_cbs[cb].congestion_window - getflightsize(cb));
    len = min(len, tcp_cbs[cb].remote_windowsize - getflightsize(cb));
    if(len <= 0){
        return 0;
    }
    
    int remaininglength = totallen -len;
    
    char buffer[ETHERNETMTU]; 
    bzero(&buffer, ETHERNETMTU);
    
    struct ipheader *ip = (struct ipheader *) buffer; 
    struct tcp *tcp = (struct tcp *) (ip + 1);
    char* data = (char *) (tcp + 1);
    
    build_ip_packet(ip, 
                    IPPROTO_TCP, 
                    tcp_cbs[cb].mcap_address, 
                    tcp_cbs[cb].dest_address, 
                    sizeof(struct tcp) + len);
    
    build_tcp_packet(tcp, 
                     tcp_cbs[cb].sourceport, 
                     tcp_cbs[cb].dest_port, 
                     tcp_cbs[cb].nextseq, 
                     tcp_cbs[cb].lastack);
    
    tcp_cbs[cb].nextseq = nextnum(tcp_cbs[cb].nextseq, len);
    
    setflag(tcp, TH_ACK);
    setflag(tcp, TH_PUSH);
    
    memcpy(data, buf, len);
    
    tcp->th_sum = tcp_checksum(ip, tcp, len);
    send_ip_packet(ip); 
    addtimeout(cb, DATA, ip);
    if(remaininglength > 0){
        return tcpwrite(cb, buf+len, remaininglength) + len;
    }
    else
        return len; 
}


void sendfinackreply(int cb){
    char buf[ETHERNETMTU]; 
    bzero(&buf, ETHERNETMTU);
    
    
    if(tcp_cbs[cb].status != CLOSE_WAIT){
        printf("Wrong state in send fin ack\n");
        return;
    }
    
    struct ipheader *ip = (struct ipheader *) buf; 
    struct tcp *tcp = (struct tcp *) (ip + 1);
    
    build_ip_packet(ip, 
                    IPPROTO_TCP, 
                    tcp_cbs[cb].mcap_address, 
                    tcp_cbs[cb].dest_address,  
                    sizeof(struct tcp));
    
  
    
   
    tcp_cbs[cb].lastack = nextnum(tcp_cbs[cb].lastack, 1);
    
    

    
    
    build_tcp_packet(tcp, 
                     tcp_cbs[cb].sourceport, 
                     tcp_cbs[cb].dest_port, 
                     tcp_cbs[cb].nextseq, 
                     tcp_cbs[cb].lastack);
    
    setflag(tcp, TH_FIN);
    setflag(tcp, TH_ACK);
    
    tcp_cbs[cb].nextseq = nextnum(tcp_cbs[cb].nextseq, 1);
    
    tcp->th_sum = tcp_checksum(ip, tcp, 0);
    send_ip_packet(ip); 
}



void sendfinack(int cb){
    char buf[ETHERNETMTU]; 
    bzero(&buf, ETHERNETMTU);
    
    if(tcp_cbs[cb].status == ESTABLISHED && tcp_cbs[cb].sockmandapi_id != -1){
        tcp_cbs[cb].status = FIN_WAIT_1;
        printf("Setting state FIN_WAIT_1\n");
    }
    else{
        //This app/socket has migrated away
        return;
    }
    
    struct ipheader *ip = (struct ipheader *) buf; 
    struct tcp *tcp = (struct tcp *) (ip + 1);
    
    build_ip_packet(ip, 
                    IPPROTO_TCP, 
                    tcp_cbs[cb].mcap_address, 
                    tcp_cbs[cb].dest_address,  
                    sizeof(struct tcp));
    

    build_tcp_packet(tcp, 
                     tcp_cbs[cb].sourceport, 
                     tcp_cbs[cb].dest_port, 
                     tcp_cbs[cb].nextseq, 
                     tcp_cbs[cb].lastack);
    
    setflag(tcp, TH_FIN);
    setflag(tcp, TH_ACK);
    
    tcp_cbs[cb].nextseq = nextnum(tcp_cbs[cb].nextseq, 1);
    
    tcp->th_sum = tcp_checksum(ip, tcp, 0);
    send_ip_packet(ip); 
}

int closeconn(int cb){
    if(tcp_cbs[cb].status == ESTABLISHED)
        sendfinack(cb);
    return 0;
}

void handlefinack(struct ipheader* ip){
    int cb = get_tcp_cb(ip);
    struct tcp  *tcp = (struct tcp *) (ip + 1);

    
   
    if(!(tcp->th_seq == tcp_cbs[cb].lastack)){
        printf("Wrong sequenze or acknumbers when handling finack\n");
        //Send rst perhaps
        return;
    }
    
    tcp_cbs[cb].lastremoteseq = tcp->th_seq;
    tcp_cbs[cb].lastremoteack = tcp->th_ack;
    
    if(tcp_cbs[cb].status == ESTABLISHED){
        ack(cb, 1);
        tcp_cbs[cb].status = CLOSE_WAIT;
        printf("Status set to CLOSE_WAIT\n");
        
        //Should ideally wait for the app to call close
        sendfinackreply(cb);
        tcp_cbs[cb].status = LAST_ACK;
        printf("Status set to LAST_ACK\n");
    }
    else if(tcp_cbs[cb].status == FIN_WAIT_2){
        tcp_cbs[cb].status = TIME_WAIT;
        printf("Status set to TIME_WAIT\n");
        ack(cb, 1);
        close_connection(cb);
        freedatastructures(cb);
    }
}



void removetimeouts(int cb){
    struct timeouts* t = tcp_cbs[cb].packets;
    while(t!=0){
        struct ipheader *ip = tcp_cbs[cb].packets->ip;
        struct tcp  *tcp = (struct tcp *) (ip + 1);
        
        int tmp = htonl(ntohl(tcp_cbs[cb].lastremoteack)-1);
        if(!between(tmp, tcp->th_seq, tcp_cbs[cb].nextseq)){
            tcp_cbs[cb].packets = t->next;
            free(t->ip);
            free(t);
            t = t->next;
        }
        else
            return;
    }
}

void update_all_rto(int cb){
    /*When an ACK is received that acknowledges new data, restart the
     *         retransmission timer so that it will expire after RTO seconds
     *         (for the current value of RTO).*/
    long int y= 0;
    long int newtimeout = getmillis() + tcp_cbs[cb].RTO;
    struct timeouts* t = tcp_cbs[cb].packets;
    while(t!=0){
        t->timesout = newtimeout;
        t = t->next;
        y++;
    }
}


void handleack(int cb, struct tcp *tcp){  
    if(tcp_cbs[cb].status == ESTABLISHED){
        tcp_cbs[cb].lastremoteack = tcp->th_ack;
        removetimeouts(cb);
        update_all_rto(cb);
    }
    else if(tcp_cbs[cb].status == FIN_WAIT_1){
        tcp_cbs[cb].lastremoteack = tcp->th_ack;
        tcp_cbs[cb].status = FIN_WAIT_2;
        printf("Setting status FIN_WAIT_2\n");

    }
    else if(tcp_cbs[cb].status == LAST_ACK){
        tcp_cbs[cb].lastremoteack = tcp->th_ack;
        tcp_cbs[cb].status = TCPCLOSED;
        printf("Setting status TCPCLOSED\n");
        close_connection(cb);
    }
    else if(tcp_cbs[cb].status == SYN_RECEIVED){
        tcp_cbs[cb].lastremoteack = tcp->th_ack;
        long int now = getmillis(); 
        tcp_cbs[cb].status = ESTABLISHED;
        printf("Setting status ESTABLISHED\n");

        if(!is_my_address(tcp_cbs[cb].mcap_address)){
            send_socket_state((struct connectioninfo *)&tcp_cbs[cb], tcp_cbs[cb].mcap_address);
        }
        
        int R = now - tcp_cbs[cb].tmp;
    
        tcp_cbs[cb].rtt = R;
        int SRTT = R;
        int RTTVAR = R/2;
        tcp_cbs[cb].RTO = SRTT + max(CLOCKGRANULARITY, 4*RTTVAR);

        if(tcp_cbs[cb].RTO < 100)
            tcp_cbs[cb].RTO = 100;
        
        doaccept(tcp_cbs[cb].sockmandapi_id);
    }
    else{
        printf("An unexpected ack receieved in handleack()\n");
    }
}


void handlenormalpacket(struct ipheader* ip){
    struct tcp  *tcp = (struct tcp *) (ip + 1);
    int cb = get_tcp_cb(ip);
    tcp_cbs[cb].remote_windowsize = ntohs(tcp->th_win);
    
    if(cb!=-1 && getflightsize(cb)>0 && tcp->th_flags & TH_ACK && tcp_cbs[cb].status == ESTABLISHED){
        if(tcp_cbs[cb].lastremoteack == tcp->th_ack){
            tcp_cbs[cb].dupacks++;

            if(tcp_cbs[cb].dupacks == 3){
                tcp_cbs[cb].sstresh = max(getflightsize(cb) / 2 , 2 * SMSS);
                send_ip_packet(tcp_cbs[cb].packets->ip);
                
                tcp_cbs[cb].congestion_window = tcp_cbs[cb].sstresh + 3*SMSS;
            }
            else if(tcp_cbs[cb].dupacks >3){
                tcp_cbs[cb].congestion_window += SMSS;
                send_ip_packet(tcp_cbs[cb].packets->ip);
            }   
        }
        else if(between(tcp_cbs[cb].lastremoteack,tcp->th_ack, tcp_cbs[cb].nextseq)){ 
            if(tcp_cbs[cb].dupacks >=3){
                tcp_cbs[cb].dupacks = 0;
                tcp_cbs[cb].congestion_window = tcp_cbs[cb].sstresh;
                handleack(cb, tcp);
            }
            else{
                if(tcp_cbs[cb].congestion_window < tcp_cbs[cb].sstresh){
                    //Slow start
                    tcp_cbs[cb].congestion_window += SMSS;
                    handleack(cb, tcp);
                }
                else{
                    //Congestion avoidance
                    handleack(cb, tcp);
                    tcp_cbs[cb].congestion_window += max(SMSS*SMSS/tcp_cbs[cb].congestion_window, 1);
                    
                }
            }
        }
        else{
            fprintf(stderr,"Unexpected, should not happen\n");
            return;

        }
    }
    else if(cb!=-1 && tcp->th_flags & TH_ACK && 
        (tcp_cbs[cb].status == FIN_WAIT_1 || 
         tcp_cbs[cb].status == LAST_ACK || 
         tcp_cbs[cb].status == SYN_RECEIVED)){
        
        handleack(cb, tcp);
    }
  

    
    unsigned short int payloadlen = ip->ip_len - (sizeof (struct ipheader)) - (tcp->tcph_offset * 4);

    if(payloadlen == 0){
        return; //No data, return
    }
    
     if(tcp->th_seq != tcp_cbs[cb].lastack){
            ack(cb, 0); 
            //Sending a dupack 
            //TODO can buffer the packet that arrived out of order
            return;
        }
        
        
        //Deliver data
     tcp_cbs[cb].lastremoteseq = tcp->th_seq;
     char * data = ((char*)tcp) + (tcp->tcph_offset * 4);
        
        if(tcp_cbs[cb].sockmandapi_id != -1){ //App is present
            deliver_tcp_data(tcp_cbs[cb].sockmandapi_id, data, payloadlen);        
            //Deliver any buffered data        
            ack(cb, payloadlen);
        }
    else{
        //Will not actually be acked until app is here
            tcp_cbs[cb].lastack = nextnum(tcp_cbs[cb].lastremoteseq, payloadlen);
            struct bufferedpacket *t = malloc(sizeof(struct bufferedpacket));
            t->next = 0;
            t->ip = malloc(ip->ip_len);
            memcpy(t->ip, ip, ip->ip_len);
            
            if(tcp_cbs[cb].bufferedpacketsfirst == 0){
                tcp_cbs[cb].bufferedpacketsfirst = t;
                tcp_cbs[cb].bufferedpacketslast = t;
            }
            
            else{
                tcp_cbs[cb].bufferedpacketslast->next = t;
                tcp_cbs[cb].bufferedpacketslast = t;
            }
        }
}

int tcplisten(int cb){
    if(tcp_cbs[cb].status != BOUND){
        return -1;
    }
    //Set status to listen    
    tcp_cbs[cb].status = TCPLISTEN;
    
    if(!is_my_address(tcp_cbs[cb].mcap_address)){
        send_socket_state((struct connectioninfo *)&tcp_cbs[cb], tcp_cbs[cb].mcap_address);
    }
    else{
        register_tcp_capture(tcp_cbs[cb].sourceport);
    }
    return 0;
}

void sendsynack(int cb){      
    char buf[ETHERNETMTU]; 
    bzero(&buf, ETHERNETMTU);
    
    struct ipheader *ip = (struct ipheader *) buf;  
    struct tcp *tcp = (struct tcp *) (ip + 1);


    //reply remote ack + 1
    build_ip_packet(ip, 
                    IPPROTO_TCP, 
                    tcp_cbs[cb].mcap_address,
                    tcp_cbs[cb].dest_address,  
                    sizeof(struct tcp));
    
    //Create random seqnum
    srandom(time(0));
    tcp_cbs[cb].nextseq = random();
    tcp_cbs[cb].lastack =  nextnum(tcp_cbs[cb].lastremoteseq, 1);
    
    
    build_tcp_packet(tcp,                        
                     tcp_cbs[cb].sourceport, 
                     tcp_cbs[cb].dest_port,
                     tcp_cbs[cb].nextseq, 
                     tcp_cbs[cb].lastack);
    
    tcp_cbs[cb].nextseq = nextnum(tcp_cbs[cb].nextseq, 1);
    
    setflag(tcp,TH_SYN);
    setflag(tcp,TH_ACK);
    tcp->th_sum = tcp_checksum(ip, tcp, 0);

    tcp_cbs[cb].RTO = 1000; //Initial value defined by rfc6298.txt
    tcp_cbs[cb].tmp = getmillis(); //Used to meassure rtt
    
    send_ip_packet(ip);
}

void replyrst(struct ipheader* oldip){
    struct tcp *oldtcp = (struct tcp *) (oldip + 1);
    
    char buf[ETHERNETMTU]; 
    bzero(&buf, ETHERNETMTU);
    
    struct ipheader *ip = (struct ipheader *) buf;  
    struct tcp *tcp = (struct tcp *) (ip + 1);
    
    build_ip_packet(ip, IPPROTO_TCP, oldip->ip_dst, oldip->ip_src, sizeof(struct tcp));
    build_tcp_packet(tcp, 
                     oldtcp->th_dport, 
                     oldtcp->th_sport, 
                     0, 
                     0);
    
    setflag(tcp, TH_RST);    
    tcp->th_sum = tcp_checksum(ip, tcp, 0);
    send_ip_packet(ip); 

}

void handlesyn(struct ipheader* ip){
    struct tcp *tcp = (struct tcp *) (ip + 1);
    int cb = get_tcp_cb(ip);
    if(cb == -1){
        replyrst(ip);
        return;
    }
    tcp_cbs[cb].status = SYN_RECEIVED;
    tcp_cbs[cb].dest_port = tcp->th_sport;
    tcp_cbs[cb].dest_address =  ip->ip_src;
    tcp_cbs[cb].congestion_window = INITIALWINDOW;
    tcp_cbs[cb].lastremoteseq = tcp->th_seq;

    sendsynack(cb);
    
}


void handlesynack(struct ipheader* oldip){
    long int now = getmillis(); 
    char buf[ETHERNETMTU]; 
    bzero(&buf, ETHERNETMTU);
    
    struct ipheader *ip = (struct ipheader *) buf; 
    struct tcp *tcp = (struct tcp *) (ip + 1);
    struct tcp *oldtcp = (struct tcp *) (oldip + 1);
    
    if(nextnum(tcp_cbs[get_tcp_cb(oldip)].nextseq, 1) != oldtcp->th_ack){
        printf("Wrong acknum from remote host in SYN/ACK msg\n");
        replyrst(oldip);
        return;
    }
    
    int cb = get_tcp_cb(oldip);
    
    tcp_cbs[cb].status = ESTABLISHED;
    tcp_cbs[cb].lastremoteseq = oldtcp->th_seq;
    tcp_cbs[cb].lastremoteack = oldtcp->th_ack;
    tcp_cbs[cb].nextseq = nextnum(tcp_cbs[cb].nextseq, 1);
    tcp_cbs[cb].lastack =  nextnum(tcp_cbs[cb].lastremoteseq, 1);
    tcp_cbs[cb].remote_windowsize = ntohs(oldtcp->th_win);
    
    build_ip_packet(ip, IPPROTO_TCP, oldip->ip_dst, oldip->ip_src, sizeof(struct tcp));
    
    build_tcp_packet(tcp, 
                     oldtcp->th_dport, 
                     oldtcp->th_sport, 
                     tcp_cbs[cb].nextseq, 
                     tcp_cbs[cb].lastack);
    
    setflag(tcp, TH_ACK);
    
    tcp->th_sum = tcp_checksum(ip, tcp, 0);
    
    connect_done(tcp_cbs[cb].sockmandapi_id, 0);
    
    int R = now - tcp_cbs[cb].tmp;
    
    tcp_cbs[cb].rtt = R;
    int SRTT = R;
    int RTTVAR = R/2;
    tcp_cbs[cb].RTO = SRTT + max(CLOCKGRANULARITY, 4*RTTVAR);


    if(tcp_cbs[cb].RTO < 100)
        tcp_cbs[cb].RTO = 100;
    removetimeouts(cb);
    //No timer on this ack
    send_ip_packet(ip); 
}

int tcpbind(unsigned int mcap_address, int sourceport, int sockmandapi_id, int uuid){
    
    int i;
    for(i = 0; i < next_tcpcb; i++){
        if(tcp_cbs[i].status > BOUND && tcp_cbs[i].status < TCPCLOSED && tcp_cbs[i].sourceport == sourceport && tcp_cbs[i].mcap_address == mcap_address){
            return -1;
        }
    }
    
    tcp_cbs[next_tcpcb].status = BOUND;
    tcp_cbs[next_tcpcb].sockmandapi_id = sockmandapi_id;
    tcp_cbs[next_tcpcb].uuid = uuid;
    tcp_cbs[next_tcpcb].mcap_address = mcap_address; 
    tcp_cbs[next_tcpcb].current_address =  get_my_addr();
    tcp_cbs[next_tcpcb].sourceport = sourceport;
    tcp_cbs[next_tcpcb].protocol = IPPROTO_TCP;
    tcp_cbs[next_tcpcb].status = 0;
    tcp_cbs[next_tcpcb].dupacks = 0;
    return next_tcpcb++;
}


int tcpconnect(int cb, unsigned int destaddr, short destport){
    char buf[ETHERNETMTU]; 
    bzero(&buf, ETHERNETMTU);
    
    struct ipheader *ip = (struct ipheader *) buf; 
    struct tcp  *tcp = (struct tcp *) (ip + 1);
    
    
    tcp_cbs[cb].dest_address = destaddr;
    tcp_cbs[cb].dest_port = destport;
    tcp_cbs[cb].status = SYN_SENT;
    tcp_cbs[cb].congestion_window = INITIALWINDOW;
       
    if(!is_my_address(tcp_cbs[cb].mcap_address)){
        send_socket_state((struct connectioninfo *)&tcp_cbs[cb], tcp_cbs[cb].mcap_address);
    }
    else{
        register_tcp_capture(tcp_cbs[cb].sourceport);
    }
    
    build_ip_packet(ip, 
                    IPPROTO_TCP, 
                    tcp_cbs[cb].mcap_address,
                    tcp_cbs[cb].dest_address,  
                    sizeof(struct tcp));
    
    //Create random seqnum
    srandom(time(0));
    unsigned int seq = random();
    
    build_tcp_packet(tcp,                        
                     tcp_cbs[cb].sourceport, 
                     tcp_cbs[cb].dest_port,
                     seq, 
                     htonl(0));
    
    setflag(tcp,TH_SYN);
    
    tcp_cbs[cb].nextseq = seq;
    
    
    tcp_cbs[cb].RTO = 1000; //Initial value defined by rfc6298.txt
    tcp_cbs[cb].tmp = getmillis(); //Used to meassure rtt
    
    
    tcp->th_sum = tcp_checksum(ip, tcp, 0);
    
    send_ip_packet(ip);
    addtimeout(cb, SYN, ip);
    return 0;
    
}



int k = 0;

void checktcpblocktimeouts(int cb){
    //Resending algorithm defined by RFC2988
    
    if(tcp_cbs[cb].packets == 0){
        return;//If there are no packets, return
    }
    
    long int now = getmillis();
    struct timeouts* t = tcp_cbs[cb].packets;
    
    if(t->timesout < now){
        struct ipheader *ip = t->ip;
        switch(t->packettype){
            case SYN:
                if(t->resent_times >= TCPMAXSYNRETRANSMISSIONS){
                    close_connection(cb);
                    printf("Resent syn too much-\n");
                    return;
                }
                else{
                    t->resent_times++;
                    t->timesout = getmillis() + tcp_cbs[cb].RTO;
                    printf("Resending syn %d\n", ++k);
                    send_ip_packet(ip);
                }
                break;
                
            case DATA:
                if(t->resent_times >= TCPMAXDATARETRANSMISSIONS){
                    printf("Resent data too much\n");
                    close_connection(cb);
                    return;
                }
                else{
                    t->resent_times++;
                    t->timesout = getmillis() + tcp_cbs[cb].RTO;

                    tcp_cbs[cb].sstresh = max(getflightsize(cb) / 2, 2 * SMSS);
                    send_ip_packet(ip);

                }                    
                break;
        }
    }
}


void checktcptimeout(){
    int i;
    for(i = 0; i < next_tcpcb; i++){
        if(tcp_cbs[i].status == ESTABLISHED || tcp_cbs[i].status == SYN_SENT){
            checktcpblocktimeouts(i);
        }
    }
}

int get_serialized_timeout_length(struct tcp_cb* cb){
    int size = 0;
    struct timeouts* t = cb->packets;
    if(t==0)
        return 0;
    while(t!=0){
        size += sizeof(struct timeouts);
        size += t->ip->ip_len;
    }    
    return size;
}

void serialize_timeouts(char* buf, int cb){
    struct timeouts* t = tcp_cbs[cb].packets;
    
    while(t!=0){
        memcpy(buf, t, sizeof(struct timeouts));
        buf += sizeof(struct timeouts);
        memcpy(buf, t->ip, t->ip->ip_len); 
        buf += t->ip->ip_len;
        t = t->next;
    }    
}

void deserialize_timeouts(char* buf, int cb){
    struct timeouts* t = (struct timeouts*)buf;
    struct timeouts* prev = 0;
    while(1){
        int last = 0;
        if(t->next==0)
            last = 1;
        
        struct timeouts* tmp = malloc(sizeof(struct timeouts));
        tmp->next = 0;
        int iplen = t->ip->ip_len;
        tmp->ip = malloc(iplen);
        memcpy(tmp, t, sizeof(struct timeouts));
        t += sizeof(struct timeouts);
        memcpy(tmp->ip, t, iplen); 
        t += iplen;
        
        tmp->timesout = getmillis() + tcp_cbs[cb].RTO;
        
        if(prev == 0)
            tcp_cbs[cb].packets = tmp;
        else
            prev->next = tmp;
        
        prev = tmp;
        
        if(last)
            break;
    }    
}

void handlerst(struct ipheader* ip){
    printf("Reset packet\n");
    int cb = get_tcp_cb(ip);
    close_connection(cb);
}


void tcpread(struct ipheader* ip){
   
    struct tcp *tcp = (struct tcp *) (ip + 1);
    
    if(tcp->th_flags & TH_SYN && tcp->th_flags & TH_ACK){ //SYNACK
        handlesynack(ip);
    }
    else if(tcp->th_flags & TH_FIN && tcp->th_flags & TH_ACK){ //FINACK
        handlefinack(ip);
    }
    else if(tcp->th_flags & TH_RST ){  
        handlerst(ip);
    }
    else if(tcp->th_flags & TH_SYN ){ 
        handlesyn(ip);
    }
    else{       
        handlenormalpacket(ip);
    }
}
