/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#define BOUND 0
#define TCPLISTEN 1
#define SYN_SENT 2
#define SYN_RECEIVED 3
#define ESTABLISHED 4
#define FIN_WAIT_1 5
#define FIN_WAIT_2 6
#define CLOSE_WAIT 7
#define LAST_ACK 8
#define TIME_WAIT 9 //Same as closed, can optimize by adding timeout closing
#define TCPCLOSED 9

#define TCPMAXSYNRETRANSMISSIONS 3
#define TCPMAXDATARETRANSMISSIONS 10
#define CLOCKGRANULARITY 1 //In ms

#define SYN 1
#define DATA 2

#define SMSS 1400
#define INITIALWINDOW 2*SMSS

struct ipheader;
struct bufferedpacket;

struct tcp_cb{
    int uuid;
    int sockmandapi_id;
    int protocol;
    short status;
    unsigned short sourceport; //in network format
    unsigned int current_address;
    unsigned int mcap_address;
    
    unsigned int dest_address; 
    unsigned short dest_port; //in network format
    
    unsigned int nextseq; //Next seqnum
    unsigned int lastack; //That we sent
    unsigned int lastremoteseq; //That we received
    unsigned int lastremoteack; //That we received
    unsigned short remote_windowsize;
    unsigned short congestion_window;
    int dupacks;
    unsigned short sstresh;

    int rtt; //Round trip time
    long int tmp; //Used to calculate rtt
    int RTO; //Retransmission timeout in milliseconds
    struct timeouts* packets; //points to the first packet in the linked list
    struct bufferedpacket* bufferedpacketsfirst;
    struct bufferedpacket* bufferedpacketslast;
};



int tcpbind(unsigned int anchor_address, int sourceport, int appapi_id, int uuid);
int tcpconnect(int cb, unsigned int destaddr, short destport);
int tcpwrite(int cb, const void *buf, int len);
void tcpread(struct ipheader* ip);
void sendfinack(int sd);
void tcpwritetest(int sd);
void checktcptimeout();

int tcp_get_cb_by_uuid(int uuid);
unsigned int tcp_get_app_location(struct ipheader *ip);
int tcp_is_app_present(struct ipheader *ip);
void* tcp_get_connection_info_by_uuid(int uuid);
int tcp_importstate(struct tcp_cb *c);
void tcp_deliverbufferdpackets(int cb);
int get_cb_by_pointer(void* p);

int get_serialized_timeout_length(struct tcp_cb* cb);
void serialize_timeouts(char* buf, int cb);
void deserialize_timeouts(char* buf, int cb);
int tcplisten(int cb);



struct timeouts{
    long int timesout;
    short resent_times; //Number of times it has been resent
    short packettype;
    struct ipheader* ip; //Pinter to the packet
    struct timeouts* next; //0 if last
};

void* get_tcp_cb_pointer(int id);

