/*
 * This file is part of SOCKMAND.
 * 
 * SOCKMAND is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SOCKMAND is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with SOCKMAND.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

#include "utils.h"
#include "headers.h"

#include <netinet/in.h>
#include <arpa/inet.h>
#include <stdlib.h>
#include <fcntl.h>
#include <time.h>
#include <sys/time.h>

#include <stdio.h>      
#include <sys/types.h>
#include <ifaddrs.h>
#include <string.h> 

int is_tcp(struct ipheader* iph){
    if(iph->ip_p == IPPROTO_TCP){
        return 1;
    }
    return 0;
}


int is_udp(struct ipheader* iph){
    if(iph->ip_p == IPPROTO_UDP){
        return 1;
    }
    return 0;
}

int is_my_address(unsigned int addr){
    if(addr == get_my_addr())
        return TRUE;
    return FALSE;
    
}

char* ip_to_char(unsigned int ip){
    struct in_addr a;
    a.s_addr = ip;
    return inet_ntoa(a);
}

char* get_readable_dst(struct ipheader* iph){
    return ip_to_char(iph->ip_dst);
}

char* get_readable_src(struct ipheader* iph){
    return ip_to_char(iph->ip_src);
}

int ipchar_to_ipuint(char* ip, unsigned int *ipuint){
    struct in_addr tmp;
    int ret = inet_aton(ip, &tmp);
    *ipuint = tmp.s_addr;
    return ret;
}


unsigned short csum(unsigned short *buf, int nwords){       
    unsigned long sum;
    for(sum=0; nwords>0; nwords--)
        sum += *buf++;
    sum = (sum >> 16) + (sum &0xffff);
    sum += (sum >> 16);
    return (unsigned short)(~sum);
}





unsigned short tcp_sum_calc(unsigned short len_tcp, unsigned short src_addr[],unsigned short dest_addr[], unsigned short buff[])
{
    /* Source: http://www.bloof.de/tcp_checksumming
     * 
     * I hope these functions helped you. 
     * Feel free to do with them wherever you want. If you post it somewhere, 
     * it would be nice of you to set a link to this tutorial so I get a few visitors on my site =)
     * 
     */
    
    unsigned char prot_tcp=6;
    unsigned long sum;
    int nleft;
    unsigned short *w;
    
    sum = 0;
    nleft = len_tcp;
    w=buff;
    
    /* calculate the checksum for the tcp header and payload */
    while(nleft > 1)
    {
        sum += *w++;
        nleft -= 2;
    }
    
    /* if nleft is 1 there ist still on byte left. We add a padding byte (0xFF) to build a 16bit word */
    if(nleft>0)
    {
        /* sum += *w&0xFF; */
        sum += *w&ntohs(0xFF00);   /* Thanks to Dalton */
    }
    
    /* add the pseudo header */
    sum += src_addr[0];
    sum += src_addr[1];
    sum += dest_addr[0];
    sum += dest_addr[1];
    sum += htons(len_tcp);
    sum += htons(prot_tcp);
    
    // keep only the last 16 bits of the 32 bit calculated sum and add the carries
    sum = (sum >> 16) + (sum & 0xFFFF);
    sum += (sum >> 16);
    
    // Take the one's complement of sum
    sum = ~sum;
    
    return ((unsigned short) sum);
}


unsigned short tcp_checksum(struct ipheader* ip,struct tcp* tcph, int size){
    return tcp_sum_calc((unsigned short)  sizeof(struct tcp) + size, (unsigned short *) &ip->ip_src, (unsigned short *) &ip->ip_dst, (unsigned short *) tcph);
    
}



void setnonblocking(sock)
int sock;
{
    int opts;
    
    opts = fcntl(sock,F_GETFL);
    if (opts < 0) {
        perror("fcntl(F_GETFL)");
        exit(EXIT_FAILURE);
    }
    opts = (opts | O_NONBLOCK);
    if (fcntl(sock,F_SETFL,opts) < 0) {
        perror("fcntl(F_SETFL)");
        exit(EXIT_FAILURE);
    }
    return;
}



long int getmillis(){
    struct timeval now;
    gettimeofday( &now, 0 );
    return (now.tv_usec + 1000000 * now.tv_sec)/1000;
}


int max(int a, int b){
    if(a>b)
        return a;
    return b;
}

int min(int a, int b){
    if(a<b)
        return a;
    return b;
}

unsigned int myaddr = 0;

unsigned int get_my_addr(){
    //Returns the first found address that is not the loopback address
    if(myaddr!=0)
        return myaddr;
    
    struct ifaddrs * ifAddrStruct=NULL;
    struct ifaddrs * ifa=NULL;
    void * tmpAddrPtr=NULL;
    
    getifaddrs(&ifAddrStruct);
    
    for (ifa = ifAddrStruct; ifa != NULL; ifa = ifa->ifa_next) {
        if (ifa ->ifa_addr->sa_family==AF_INET) { // check it is IP4
                tmpAddrPtr=&((struct sockaddr_in *)ifa->ifa_addr)->sin_addr;
                char addressBuffer[INET_ADDRSTRLEN];
                inet_ntop(AF_INET, tmpAddrPtr, addressBuffer, INET_ADDRSTRLEN);
                if(strcmp(addressBuffer, "127.0.0.1")){
                    printf("%s IP Address '%s'\n", ifa->ifa_name, addressBuffer); 
                    myaddr = inet_addr(addressBuffer);
                    if (ifAddrStruct!=NULL) freeifaddrs(ifAddrStruct);
                    return myaddr;
                }   
        } 
    }
    printf("Major error, your IP address was not found\n");
    exit(-1);
    return -1;
}